/******************************************************************************
FILE:               AutoresizingTest.m
LANGUAGE:           Objective-C
SYSTEM:             POSIX
USER-INTERFACE:     OpenStep (GNUstep)
DESCRIPTION
    $Header$
    This class is the AutoresizingTest application.

    A simple application to test the autoresizing property of NSViews.

AUTHORS
    <PJB> Pascal J. Bourguignon
MODIFICATIONS
    $Log$
    2000-04-14 <PJB> Creation.
BUGS
LEGAL
    GPL
    Copyright Pascal J. Bourguignon 2000 - 2000

    This file is part of the AutoresizingTest application.

    This  program is  free software;  you can  redistribute  it and/or
    modify it  under the  terms of the  GNU General Public  License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program  is distributed in the  hope that it  will be useful,
    but  WITHOUT ANY WARRANTY;  without even  the implied  warranty of
    MERCHANTABILITY or FITNESS FOR  A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a  copy of the GNU General Public License
    along with  this program; see the  file COPYING; if  not, write to
    the Free  Software Foundation, Inc.,  59 Temple Place,  Suite 330,
    Boston, MA 02111-1307 USA
******************************************************************************/
#include "AutoresizingTest.h"
#include "Version.h"


@implementation AutoresizingTest

    const float WindowTitleHeight=20.0;



// NSObject methods:

    -(id)init
    {
        self=[super init];
        if(self!=nil){
            testWin=nil;
            testView=nil;
            cmdsWin=nil;
            [self setDelegate:self];
        }
        return(self);
    }//init;


    -(void)dealloc
    {
        [testWin close];
        [testWin release];
        [cmdsWin close];
        [cmdsWin release];
        [super dealloc];
    }//dealloc;


// NSApplication delegate methods:

    -(void)applicationDidFinishLaunching:(NSNotification*)notification
    {
        [NSApp setMainMenu:[self makeMenu]];
        [self makeCommandWindow];
        [self makeTestWindow];
        [cmdsWin orderFront:self];
        [testWin makeKeyAndOrderFront:self];
    }//applicationDidFinishLaunching:;




// AutoresizingTest actions:

    -showInformations:sender
    {
        [[NSApplication sharedApplication]
            orderFrontStandardInfoPanelWithOptions:[NSDictionary
               dictionaryWithObjects:[NSArray arrayWithObjects:
                  @"A simple application to test the autoresizing property of NSViews.",
                  [NSString stringWithCString:VERSION],
                  [NSString stringWithFormat:@"%s-gs0.6.5",VERSION],
                  [NSArray arrayWithObjects:
                      @"Pascal J. Bourguignon <pjb@imaginet.fr>",
                      nil],
                  @"Copyright  Pascal J. Bourguignon  2000",
                  @"http://www.imaginet.fr/~pjb/",
@"[GPL]\n"
@"This  program is  free software;  you can  redistribute  it and/or\n"
@"modify it  under the  terms of the  GNU General Public  License as\n"
@"published by the Free Software Foundation; either version 2 of the\n"
@"License, or (at your option) any later version.\n"
@"\n"
@"This program  is distributed in the  hope that it  will be useful,\n"
@"but  WITHOUT ANY WARRANTY;  without even  the implied  warranty of\n"
@"MERCHANTABILITY or FITNESS FOR  A PARTICULAR PURPOSE.  See the GNU\n"
@"General Public License for more details.\n"
@"\n"
@"You should have received a  copy of the GNU General Public License\n"
@"along with  this program; see the  file COPYING; if  not, write to\n"
@"the Free  Software Foundation, Inc.,  59 Temple Place,  Suite 330,\n"
@"Boston, MA 02111-1307 USA\n",
                  nil]
              forKeys:[NSArray arrayWithObjects:
                  @"ApplicationDescription",
                  @"ApplicationRelease",
                  @"FullVersionID",
                  @"Authors",
                  @"URL",
                  @"Copyright",
                  @"CopyrightDescription",
                  nil]]];
        return(self);
    }//showInformations:;


    -changeAutoresizing:sender
    {
        [self updateElasticity];
        return(self);
    }//changeAutoresizing:;


    -setAllNotElastic:sender
    {
        int i;
        for(i=0;i<6;i++){
            [elasticButtons[i] setState:0];
        }
        [self updateElasticity];
        return(self);
    }//setAllNotElastic:;


    -setAllElastic:sender
    {
        int i;
        for(i=0;i<6;i++){
            [elasticButtons[i] setState:1];
        }
        [self updateElasticity];
        return(self);
    }//setAllElastic:;


    -resetTestWin:sender
    {
        [testWin setContentSize:NSMakeSize(180.0,180.0)];
        [testView setFrame:NSMakeRect(60.0,60.0,60.0,60.0)];
        return(self);
    }//resetTestWin:;





// AutoresizingTest methods:

    -(void)updateElasticity
    {
        int mask=0;
        int i;
        for(i=0;i<6;i++){
            if([elasticButtons[i] state]!=0){
                mask|=[elasticButtons[i] tag];
            }
        }
        [testView setAutoresizingMask:mask];
    }//updateElasticity;




    static NSMenuItem* makeitem(NSMenu* menu,NSString* tit,
                                SEL sel,
                                NSString* key,
                                id tar)
    {
        NSMenuItem* item;
        item=[menu addItemWithTitle:tit action:sel keyEquivalent:key];
        [item setEnabled:YES];
        [item setTarget:tar];
        return(item);
    }//makeitem;


    -(NSMenu*)makeMenu
    {
        NSMenuItem* item;
        NSMenu*     menu;
        NSMenu*     subm;


        menu=[[NSMenu alloc] initWithTitle:[[self class]description]];
        item=makeitem(menu,@"Informations",           
                      @selector(showInformations:),  @"i",self);


        subm=[[NSMenu alloc]initWithTitle:@"Windows"];
        item=makeitem(subm,@"Miniaturize window",     
                      @selector(miniaturize:),       @"m",nil);
        item=makeitem(menu,@"Windows",                
                      @selector(submenuAction:),     @"", nil);
        [menu setSubmenu:subm forItem:item];

        item=makeitem(menu,@"Hide",                   
                      @selector(hide:),              @"h",self);
        item=makeitem(menu,@"Quit",                   
                      @selector(terminate:),         @"q",self);

        [menu sizeToFit];
        return(menu);
    }//makeMenu;


/*
     +----------+----------+----------+
     |          |          |          |
     |          |   maxY   |          |
     |          |          |          |
     +----------+----------+----------+
     |          |    height|          |
     |  minX    |          |  maxX    |
     |          |width     |          |
     +----------+----------+----------+
     |          |          |          |
     |          |   minY   |          |
     |          |          |          |
     +----------+----------+----------+

         all       none      resettest

 */


    -(NSButton*)makeButtonTitled:(NSString*)title
                            type:(NSButtonType)type
                           frame:(NSRect)frame
                             tag:(int)tag
                          action:(SEL)action
                          target:(id)target
    {
        NSButton* button=[[NSButton alloc]initWithFrame:frame];
        [button autorelease];
        [button setTitle:title];
        [button setButtonType:type];
        [button setTag:tag];
        [button setAction:action];
        [button setTarget:target];
        if(type==NSSwitchButton){
            [button setBordered:NO];
        }
        return(button);
    }//makeButtonTitled:frame:tag:action:target:;


    -(void)makeCommandWindow
    {
        NSScreen*   screen=[NSScreen mainScreen];
        NSRect      screenFrame=[screen frame];
        NSRect      winFrame=NSMakeRect(0.0,0.0,200.0,225.0);
        NSView*     content;
        NSButton*   button;
        int         i;

        content=[[NSView alloc]initWithFrame:winFrame];
        [content autorelease];

        // ------------------------------------------------------------ //

        i=0;
        button=[self makeButtonTitled:@"Min X"
                     type:NSSwitchButton
                     frame:NSMakeRect(5.0,95.0,60.0,60.0)
                     tag:NSViewMinXMargin
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;


        button=[self makeButtonTitled:@"Width"
                     type:NSSwitchButton
                     frame:NSMakeRect(70.0,95.0,60.0,30.0)
                     tag:NSViewWidthSizable
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;


        button=[self makeButtonTitled:@"Max X"
                     type:NSSwitchButton
                     frame:NSMakeRect(135.0,95.0,60.0,60.0)
                     tag:NSViewMaxXMargin
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;


        button=[self makeButtonTitled:@"Min Y"
                     type:NSSwitchButton
                     frame:NSMakeRect(70.0,30.0,60.0,60.0)
                     tag:NSViewMinYMargin
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;


        button=[self makeButtonTitled:@"Height"
                     type:NSSwitchButton
                     frame:NSMakeRect(70.0,125.0,60.0,30.0)
                     tag:NSViewHeightSizable
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;


        button=[self makeButtonTitled:@"Max Y"
                     type:NSSwitchButton
                     frame:NSMakeRect(70.0,160.0,60.0,60.0)
                     tag:NSViewMaxYMargin
                     action:@selector(changeAutoresizing:)
                     target:self];
        [content addSubview:button];
        elasticButtons[i++]=button;
    
        // ------------------------------------------------------------ //

        button=[self makeButtonTitled:@"All Not Elastic"
                     type:NSMomentaryPushButton
                     frame:NSMakeRect(5.0,5.0,60.0,20.0)
                     tag:NSViewMaxYMargin
                     action:@selector(setAllNotElastic:)
                     target:self];
        [content addSubview:button];


        button=[self makeButtonTitled:@"All Elastic"
                     type:NSMomentaryPushButton
                     frame:NSMakeRect(70.0,5.0,60.0,20.0)
                     tag:NSViewMaxYMargin
                     action:@selector(setAllElastic:)
                     target:self];
        [content addSubview:button];


        button=[self makeButtonTitled:@"Reset Test"
                     type:NSMomentaryPushButton
                     frame:NSMakeRect(135.0,5.0,60.0,20.0)
                     tag:NSViewMaxYMargin
                     action:@selector(resetTestWin:)
                     target:self];
        [content addSubview:button];

        // ------------------------------------------------------------ //

        winFrame.origin.x=screenFrame.origin.x+screenFrame.size.width
                         -winFrame.size.width-56;
        winFrame.origin.y=screenFrame.origin.x+screenFrame.size.height
                         -winFrame.size.height-WindowTitleHeight;
        [cmdsWin release];
        cmdsWin=[[NSWindow alloc]initWithContentRect:winFrame
                                 styleMask:NSTitledWindowMask
                                          |NSMiniaturizableWindowMask
                                 backing:NSBackingStoreBuffered
                                 defer:NO];
        [cmdsWin setTitle:@"Setting Window"];
        [cmdsWin setContentView:content];
    }//makeCommandWindow;


    -(void)makeTestWindow
    {
        NSRect  winFrame=NSMakeRect(0.0,0.0,180.0,180.0);
        NSRect  viewFrame=NSMakeRect(60.0,60.0,60.0,60.0);
        NSView* content;

        testView=[[NSButton alloc]initWithFrame:viewFrame];
        [testView autorelease];

        content=[[NSView alloc]initWithFrame:winFrame];
        [content autorelease];
        [content setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
        [content addSubview:testView];

        [testWin release];
        testWin=[[NSWindow alloc]initWithContentRect:winFrame
                                 styleMask:NSTitledWindowMask
                                          |NSMiniaturizableWindowMask
                                          |NSResizableWindowMask
                                 backing:NSBackingStoreBuffered
                                 defer:NO];
        [testWin setTitle:@"Test Window"];
        [testWin setContentView:content];
        [testWin center];
    }//makeTestWindow;


@end // AutoresizingTest;

/*** AutoresizingTest.m                       -- 2000-04-12 03:42:05 -- PJB ***/
